@@ -19,11 +19,8 @@
 
 #include <config.h>
 #include <panel-applet-gconf.h>
-#if HAVE_EGG
+
 #include <egg-screen-help.h>
-#else
-#include <libgnome/gnome-help.h>
-#endif
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -117,7 +114,6 @@ about_cb (BonoboUIComponent          *uic,
 	  AccessxStatusApplet        *sapplet,
 	  const gchar                *verbname)
 {
-	static GtkWidget *about = NULL;
 	GdkPixbuf	 *pixbuf;
 	GError		 *error = NULL;
 	gchar		 *file;
@@ -129,16 +125,15 @@ about_cb (BonoboUIComponent          *uic,
 	};
 
 	const gchar *documenters[] = {
+	        "Bill Haneman <bill.haneman@sun.com>",
+                "Sun GNOME Documentation Team <gdocteam@sun.com>",
 		NULL
 	};
 
 	const gchar *translator_credits = _("translator_credits");
 
-	if (about) {
-		gtk_window_set_screen (
-			GTK_WINDOW (about),
-			gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
-		gtk_window_present (GTK_WINDOW (about));
+	if (sapplet->about_dialog) {
+		gtk_window_present (GTK_WINDOW (sapplet->about_dialog));
 		return;
 	}
         
@@ -153,25 +148,24 @@ about_cb (BonoboUIComponent          *uic,
 		g_error_free (error);
 	}
 	
-        about = gnome_about_new (
-		_("AccessX Status"), VERSION,
-		_("Copyright (C) 2003 Sun Microsystems"),
-		_("Shows the state of AccessX features such as latched modifiers"),
-		authors,
-		documenters,
-		strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-		pixbuf);
+        sapplet->about_dialog = gnome_about_new (_("AccessX Status"), VERSION,
+						 _("Copyright (C) 2003 Sun Microsystems"),
+						 _("Shows the state of AccessX features such as latched modifiers"),
+						 authors,
+						 documenters,
+						 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+						 pixbuf);
 		
 	if (pixbuf)
 		gdk_pixbuf_unref (pixbuf);
 			
-	gtk_window_set_wmclass (GTK_WINDOW (about), "accessx status", "AccessX Status");
-	gtk_window_set_screen (GTK_WINDOW (about),
+	gtk_window_set_wmclass (GTK_WINDOW (sapplet->about_dialog), "accessx status", "AccessX Status");
+	gtk_window_set_screen (GTK_WINDOW (sapplet->about_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
-	g_signal_connect (about, "destroy",
+	g_signal_connect (sapplet->about_dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
-			  &about);
-        gtk_widget_show (about);
+			  &sapplet->about_dialog);
+	gtk_widget_show (sapplet->about_dialog);
 }
 
 static void
@@ -180,16 +174,13 @@ help_cb (BonoboUIComponent   *uic,
 	 const char          *verbname)
 {
 	GError *error = NULL;
-#if HAVE_EGG
+
 	egg_help_display_on_screen (
-		"accessx-help.xml", NULL,
+		"accessx-status.xml", NULL,
 		gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)),
 		&error);
+
 	if (error) { 
-#else 
-	gboolean retval = gnome_help_display ("accessx-status.xml", NULL, &error);
-	if (!retval) {
-#endif
 		GtkWidget *dialog = 
 			gtk_message_dialog_new (GTK_WINDOW (sapplet->applet),
 						GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -197,10 +188,15 @@ help_cb (BonoboUIComponent   *uic,
 						GTK_BUTTONS_CLOSE,
 						_("There was an error launching the help viewer : %s"), 
 						error->message);
+
 		g_signal_connect (G_OBJECT (dialog),
 				  "response",
 				  G_CALLBACK (gtk_widget_destroy), NULL);
+
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		
 		gtk_widget_show (dialog);
 		g_error_free (error);
 	}
@@ -212,7 +208,12 @@ dialog_cb (BonoboUIComponent *component,
 	   const char        *verb)
 {
 	GError *err = NULL;
-	if (!g_spawn_command_line_async ("gnome-accessibility-keyboard-properties", &err)) {
+	int ret;
+
+	ret = egg_screen_execute_shell (gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)),
+					NULL, "gnome-accessibility-keyboard-properties", &err);
+
+	if (ret < 0) {
 		GtkWidget *dialog = 
 			gtk_message_dialog_new (NULL,
 						GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -220,10 +221,15 @@ dialog_cb (BonoboUIComponent *component,
 						GTK_BUTTONS_CLOSE,
 						_("There was an error launching the keyboard capplet : %s"), 
 						err->message);
+
 		g_signal_connect (G_OBJECT (dialog),
 				  "response",
 				  G_CALLBACK (gtk_widget_destroy), NULL);
+
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (sapplet->applet)));
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 		gtk_widget_show (dialog);
 		g_error_free (err);
 	}
@@ -962,8 +968,8 @@ accessx_status_applet_layout_box (AccessxStatusApplet *sapplet, GtkWidget *box,
 	sapplet->box = box;
 
 	atko = gtk_widget_get_accessible (sapplet->box);
-	atk_object_set_name (atko, "AccessX Status");
-	atk_object_set_description (atko, "Shows keyboard status when accessibility features are used.");
+	atk_object_set_name (atko, _("AccessX Status"));
+	atk_object_set_description (atko, _("Shows keyboard status when accessibility features are used."));
 
 	gtk_widget_show (sapplet->box);
 	gtk_widget_show (GTK_WIDGET (sapplet->applet));
@@ -1060,8 +1066,8 @@ create_applet (PanelApplet *applet)
 
 	accessx_status_applet_layout_box (sapplet, box, stickyfoo);
 	atko = gtk_widget_get_accessible (GTK_WIDGET (sapplet->applet));
-	atk_object_set_name (atko, "AccessX Status");
-	atk_object_set_description (atko, "Shows keyboard status when accessibility features are used.");
+	atk_object_set_name (atko, _("AccessX Status"));
+	atk_object_set_description (atko, _("Shows keyboard status when accessibility features are used."));
 	return sapplet;
 }
 
@@ -1077,6 +1083,8 @@ accessx_status_applet_destroy (GtkWidget *widget, gpointer user_data)
 		XkbFreeKeyboard (sapplet->xkb, 0, True);
 	if (sapplet->xkb_display) 
 		XCloseDisplay (sapplet->xkb_display);
+	if (sapplet->about_dialog)
+		gtk_widget_destroy (sapplet->about_dialog);
 	if (sapplet->tooltips)
 		g_object_unref (sapplet->tooltips);
 }
@@ -1187,6 +1195,8 @@ accessx_status_applet_fill (PanelApplet *applet)
 					      NULL);
 	}
 
+	sapplet->about_dialog = NULL;
+
 	sapplet->tooltips = gtk_tooltips_new ();
 	g_object_ref (sapplet->tooltips);
 	gtk_object_sink (GTK_OBJECT (sapplet->tooltips));
