@@ -33,6 +33,7 @@ typedef struct
 typedef struct 
 {
 	/* Applet */
+	GtkWidget   *about_dialog;
 	PanelApplet *applet;
 	GtkWidget   *vbox;
 	GtkWidget   *hbox;
@@ -69,8 +70,10 @@ void destroy_eyes (EyesApplet *eyes_applet);
 
 
 /* theme.c */
-void load_theme    (EyesApplet        *eyes_applet,
-		    const gchar       *theme_dir);
+void theme_dirs_create ();
+
+int load_theme    (EyesApplet        *eyes_applet,
+		   const gchar       *theme_dir);
 
 void properties_cb (BonoboUIComponent *uic,
 		    EyesApplet        *eyes_applet,
