@@ -1,4 +1,4 @@
-/* $Id: weather.c,v 1.63 2003/07/16 19:21:40 baddog Exp $ */
+/* $Id: weather.c,v 1.71 2004/02/05 23:59:23 dsmit Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -37,6 +37,7 @@
 
 void
 close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data);
+static gchar* formatWeatherMsg (gchar* msg);
 
 /* FIXME: these global variables will cause conflicts when multiple
 ** instances of the applets update at the same time
@@ -138,8 +139,9 @@ static const gchar *wind_direction_str[] = {
 
 const gchar *weather_wind_direction_string (WeatherWindDirection wind)
 {
-	if (wind < 0 ||
-	    wind >= (sizeof (wind_direction_str) / sizeof (char *)))
+        if (wind < 0)
+	        return _("Unknown");
+	if (wind >= (sizeof (wind_direction_str) / sizeof (char *)))
 		return _("Invalid");
 
 	return _(wind_direction_str[(int)wind]);
@@ -263,7 +265,8 @@ static inline void requests_done_check (WeatherInfo *info)
     g_return_if_fail(info->requests_pending);
 
     if (!info->metar_handle && !info->iwin_handle && 
-        !info->wx_handle && !info->met_handle) {
+        !info->wx_handle && !info->met_handle &&
+	!info->bom_handle) {
         info->requests_pending = FALSE;
         update_finish(info);
     }
@@ -284,6 +287,8 @@ close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
 		info->wx_handle = NULL;
 	if (handle == info->met_handle)
 		info->met_handle = NULL;
+	if (handle == info->bom_handle)
+	        info->bom_handle = NULL;
  	
 	requests_done_check(info);
 		
@@ -451,7 +456,7 @@ static gboolean metar_tok_wind (gchar *tokp, WeatherInfo *info)
         gust = atoi(sgust);
     }
 
-    if ((349 <= dir) && (dir <= 11))
+    if ((349 <= dir) || (dir <= 11))
         info->wind = WIND_N;
     else if ((12 <= dir) && (dir <= 33))
         info->wind = WIND_NNE;
@@ -1047,7 +1052,7 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	
     if (result == GNOME_VFS_ERROR_EOF)
     {
-        info->forecast = g_strdup (info->iwin_buffer);
+        info->forecast = formatWeatherMsg(g_strdup (info->iwin_buffer));
     }
     else if (result != GNOME_VFS_OK) {
 	g_print("%s", gnome_vfs_result_to_string(result));
@@ -1063,6 +1068,47 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     return;
 }
 
+
+/**
+ *  Human's don't deal well with .MONDAY...SUNNY AND BLAH BLAH.TUESDAY...THEN THIS AND THAT.WEDNESDAY...RAINY BLAH BLAH.
+ *  This function makes it easier to read.
+ */
+static gchar* formatWeatherMsg (gchar* forecast) {
+
+    gchar* ptr = forecast;
+    gchar* startLine = NULL;
+
+    while (0 != *ptr) {
+        if (ptr[0] == '\n' && ptr[1] == '.') {
+            if (NULL == startLine) {
+                memmove(forecast, ptr, strlen(ptr) + 1);
+                ptr[0] = ' ';
+                ptr = forecast;
+            }
+            ptr[1] = '\n';
+            ptr += 2;
+            startLine = ptr;
+
+        } else if (ptr[0] == '.' && ptr[1] == '.' && ptr[2] == '.' && NULL != startLine) {
+            memmove(startLine + 2, startLine, (ptr - startLine) * sizeof(gchar));
+            startLine[0] = ' ';
+            startLine[1] = '\n';
+            ptr[2] = '\n';
+
+            ptr += 3;
+
+        } else if (ptr[0] == '$' && ptr[1] == '$') {
+            ptr[0] = ptr[1] = ' ';
+
+        } else {
+            ptr++;
+        }
+    }
+
+    return forecast;
+}
+
+
 static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
 {
     WeatherInfo *info = (WeatherInfo *)data;
@@ -1072,7 +1118,7 @@ static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     gchar *forecast;
 
     g_return_if_fail(handle == info->iwin_handle);
-	
+
     g_return_if_fail(info != NULL);
 
     body = g_malloc0(DATA_SIZE);
@@ -1353,6 +1399,108 @@ static void metoffice_start_open (WeatherInfo *info)
     return;
 }	
 
+static gchar *bom_parse (gchar *meto)
+{ 
+    gchar *p, *rp;
+    
+    p = strstr(meto, "<pre>");
+    p += 5; /* skip the <pre> */
+    rp = strstr(p, "</pre>");
+
+    return g_strndup(p, rp-p);
+}
+
+static void bom_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result, 
+			    gpointer buffer, GnomeVFSFileSize requested, 
+			    GnomeVFSFileSize body_len, gpointer data)
+{
+    WeatherInfo *info = (WeatherInfo *)data;
+    WeatherLocation *loc;
+    gchar *body, *forecast, *temp;
+
+    info->forecast = NULL;
+    loc = info->location;
+    body = (gchar *)buffer;
+    body[body_len] = '\0';
+	
+    if (info->bom_buffer == NULL)
+        info->bom_buffer = g_strdup(body);
+    else
+    {
+        temp = g_strdup(info->bom_buffer);
+	g_free(info->bom_buffer);
+	info->bom_buffer = g_strdup_printf("%s%s", temp, body);
+	g_free(temp);
+    }
+	
+    if (result == GNOME_VFS_ERROR_EOF)
+    {
+	forecast = bom_parse(info->bom_buffer);
+        info->forecast = forecast;
+    }
+    else if (result != GNOME_VFS_OK) {
+	info->bom_handle = NULL;
+	requests_done_check (info);
+        g_warning("Failed to get BOM data.\n");
+    } else {
+	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, bom_finish_read, info);
+
+	return;
+    }
+    
+    request_done(info->bom_handle, info);
+    g_free (buffer);
+    return;
+}
+
+static void bom_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
+{
+    WeatherInfo *info = (WeatherInfo *)data;
+    WeatherLocation *loc;
+    gchar *body;
+    gint body_len;
+    gchar *forecast;
+
+    g_return_if_fail(handle == info->bom_handle);
+
+    g_return_if_fail(info != NULL);
+
+    body = g_malloc0(DATA_SIZE);
+
+    info->bom_buffer = NULL;
+    if (info->forecast)
+    	g_free (info->forecast);	
+    info->forecast = NULL;
+    loc = info->location;
+    g_return_if_fail(loc != NULL);
+
+    if (result != GNOME_VFS_OK) {
+        g_warning("Failed to get BOM forecast data.\n");
+        info->bom_handle = NULL;
+        requests_done_check (info);
+        g_free (body);
+    } else {
+    	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, bom_finish_read, info);
+    }
+    return;
+}
+
+static void bom_start_open (WeatherInfo *info)
+{
+    gchar *url;
+    WeatherLocation *loc;
+    loc = info->location;
+  
+    url = g_strdup_printf("http://www.bom.gov.au/cgi-bin/wrap_fwo.pl?%s.txt",
+			  loc->zone+1);
+
+    gnome_vfs_async_open(&info->bom_handle, url, GNOME_VFS_OPEN_READ, 
+    			 0, bom_finish_open, info);
+    g_free(url);
+
+    return;
+}	
+
 /* Get forecast into newly alloc'ed string */
 static void iwin_start_open (WeatherInfo *info)
 {
@@ -1371,6 +1519,11 @@ static void iwin_start_open (WeatherInfo *info)
     	metoffice_start_open (info);
     	return;
     }
+    if (loc->zone[0] == '@')    /* Australian BOM forecasts */
+    {
+    	bom_start_open (info);
+    	return;
+    }
     
 #if 0
     if (weather_forecast == FORECAST_ZONE)
@@ -1386,7 +1539,7 @@ static void iwin_start_open (WeatherInfo *info)
     ** as pa, and the zone as lower case paz021.
     */
     zone = g_strdup (loc->zone);
-    g_strdown (zone);
+    g_ascii_strdown (zone, strlen (zone));
     state = g_strdup (zone);
     state[2] = '\0';
     
@@ -1523,6 +1676,7 @@ gboolean _weather_info_fill (GWeatherApplet *applet, WeatherInfo *info, WeatherL
     	info->iwin_handle = NULL;
     	info->wx_handle = NULL;
     	info->met_handle = NULL;
+	info->bom_handle = NULL;
     	info->requests_pending = FALSE;
     	info->metar_buffer = NULL;
     	info->iwin_buffer = NULL; 
@@ -1545,18 +1699,20 @@ gboolean _weather_info_fill (GWeatherApplet *applet, WeatherInfo *info, WeatherL
     }
 
     /* Defaults (just in case...) */
+    /* Well, no just in case anymore.  We may actually fail to fetch some
+     * fields. */
     info->units = UNITS_IMPERIAL;
     info->update = 0;
-    info->sky = SKY_CLEAR;
+    info->sky = -1;
     info->cond.significant = FALSE;
     info->cond.phenomenon = PHENOMENON_NONE;
     info->cond.qualifier = QUALIFIER_NONE;
-    info->temp = 0;
-    info->dew = 0;
-    info->humidity = 0;
-    info->wind = WIND_VARIABLE;
-    info->windspeed = 0;
-    info->pressure = 0.0;
+    info->temp = -1000.0;
+    info->dew = -1000.0;
+    info->humidity = -1;
+    info->wind = -1;
+    info->windspeed = -1;
+    info->pressure = -1.0;
     info->visibility = -1.0;
     info->forecast = NULL;
     info->radar = NULL;
@@ -1568,6 +1724,7 @@ gboolean _weather_info_fill (GWeatherApplet *applet, WeatherInfo *info, WeatherL
     info->iwin_handle = NULL;
     info->wx_handle = NULL;
     info->met_handle = NULL;
+    info->bom_handle = NULL;
     info->requests_pending = TRUE;
     info->applet = applet;
 
@@ -1631,17 +1788,17 @@ WeatherInfo *weather_info_config_read (PanelApplet *applet)
     info->location = weather_location_config_read(applet);
     info->units = (WeatherUnits)0;
     info->update = (WeatherUpdate)0;
-    info->sky = (WeatherSky)0;
+    info->sky = -1;
     info->cond.significant = FALSE;
     info->cond.phenomenon = (WeatherConditionPhenomenon)0;
     info->cond.qualifier = (WeatherConditionQualifier)0;
-    info->temp = 0;
-    info->dew = 0;
-    info->humidity = 0;
-    info->wind = (WeatherWindDirection)0;
-    info->windspeed = 0;
-    info->pressure = 0;
-    info->visibility = 0;
+    info->temp = -1000.0;
+    info->dew = -1000.0;
+    info->humidity = -1;
+    info->wind = -1;
+    info->windspeed = -1;
+    info->pressure = -1.0;
+    info->visibility = -1.0;
     info->forecast = g_strdup ("None");
     info->radar = NULL;  /* FIX */
     info->requests_pending = FALSE;
@@ -1791,6 +1948,8 @@ const gchar *weather_info_get_sky (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->sky < 0)
+	return "Unknown";
     return weather_sky_string(info->sky);
 }
 
@@ -1808,6 +1967,8 @@ const gchar *weather_info_get_temp (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->temp < -500.0)
+        return _("Unknown");
     g_snprintf(buf, sizeof (buf), "%.1f%s", info->temp, TEMP_UNIT_STR(info->units));
     return buf;
 }
@@ -1818,6 +1979,8 @@ const gchar *weather_info_get_dew (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->dew < -500.0)
+        return _("Unknown");
     g_snprintf(buf, sizeof (buf), "%.1f%s", info->dew, TEMP_UNIT_STR(info->units));
     return buf;
 }
@@ -1828,6 +1991,8 @@ const gchar *weather_info_get_humidity (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->humidity < 0.0)
+        return _("Unknown");
     g_snprintf(buf, sizeof (buf), "%d%%", info->humidity);
     return buf;
 }
@@ -1838,6 +2003,8 @@ const gchar *weather_info_get_wind (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->windspeed < 0.0 || info->wind < 0)
+        return _("Unknown");
     if (info->windspeed == 0.00) {
         strncpy(buf, _("Calm"), sizeof(buf));
 	buf[sizeof(buf)-1] = '\0';
@@ -1854,6 +2021,8 @@ const gchar *weather_info_get_pressure (WeatherInfo *info)
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
+    if (info->pressure < 0.0)
+        return _("Unknown");
     g_snprintf(buf, sizeof (buf), "%.2f %s", info->pressure, PRESSURE_UNIT_STR(info->units));
     return buf;
 }
@@ -1889,7 +2058,7 @@ const gchar *weather_info_get_temp_summary (WeatherInfo *info)
     
     if (!info)
         return NULL;
-    if (!info->valid)
+    if (!info->valid || info->temp < -500.0)
         return "--";
     
     if (!info->applet)
@@ -1899,7 +2068,8 @@ const gchar *weather_info_get_temp_summary (WeatherInfo *info)
     else                                                                    
       degree = "F";
       
-   g_snprintf(buf, sizeof (buf), "%d\302\260%s", (int)(info->temp + 0.5), degree);
+   g_snprintf(buf, sizeof (buf), "%d\302\260%s",
+	      (int)floor(info->temp + 0.5), degree);
     
     return buf;
 }
@@ -1920,24 +2090,6 @@ gchar *weather_info_get_weather_summary (WeatherInfo *info)
 static GdkPixbuf **weather_pixbufs_mini = NULL;
 static GdkPixbuf **weather_pixbufs = NULL;
 
-#include "pixmaps/unknown-mini.xpm"
-#include "pixmaps/sun-mini.xpm"
-#include "pixmaps/suncloud-mini.xpm"
-#include "pixmaps/cloud-mini.xpm"
-#include "pixmaps/rain-mini.xpm"
-#include "pixmaps/tstorm-mini.xpm"
-#include "pixmaps/snow-mini.xpm"
-#include "pixmaps/fog-mini.xpm"
-
-#include "pixmaps/unknown.xpm"
-#include "pixmaps/sun.xpm"
-#include "pixmaps/suncloud.xpm"
-#include "pixmaps/cloud.xpm"
-#include "pixmaps/rain.xpm"
-#include "pixmaps/tstorm.xpm"
-#include "pixmaps/snow.xpm"
-#include "pixmaps/fog.xpm"
-
 #define PIX_UNKNOWN   0
 #define PIX_SUN       1
 #define PIX_SUNCLOUD  2
@@ -1969,30 +2121,33 @@ xpm_to_pixmap (char **xpm_data, GdkPixmap **pixmap, GdkBitmap **mask)
 static void init_pixbufs (void)
 {
     static gboolean initialized = FALSE;
+    GtkIconTheme *icon_theme;
 
     if (initialized)
        return;
     initialized = TRUE;
 
+    icon_theme = gtk_icon_theme_get_default ();
+
     weather_pixbufs_mini = g_new(GdkPixbuf *, NUM_PIX);
-    weather_pixbufs_mini[PIX_UNKNOWN] = gdk_pixbuf_new_from_xpm_data ((const char **)unknown_mini_xpm);	
-    weather_pixbufs_mini[PIX_SUN] = gdk_pixbuf_new_from_xpm_data ((const char **)sun_mini_xpm);	
-    weather_pixbufs_mini[PIX_SUNCLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)suncloud_mini_xpm);	
-    weather_pixbufs_mini[PIX_CLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)cloud_mini_xpm);	
-    weather_pixbufs_mini[PIX_RAIN] = gdk_pixbuf_new_from_xpm_data ((const char **)rain_mini_xpm);	
-    weather_pixbufs_mini[PIX_TSTORM] = gdk_pixbuf_new_from_xpm_data ((const char **)tstorm_mini_xpm);	
-    weather_pixbufs_mini[PIX_SNOW] = gdk_pixbuf_new_from_xpm_data ((const char **)snow_mini_xpm);	
-    weather_pixbufs_mini[PIX_FOG] = gdk_pixbuf_new_from_xpm_data ((const char **)fog_mini_xpm);	
+    weather_pixbufs_mini[PIX_UNKNOWN] = gtk_icon_theme_load_icon (icon_theme, "stock_unknown", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_SUN] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-sunny", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_SUNCLOUD] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-few-clouds", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_CLOUD] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-cloudy", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_RAIN] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-showers", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_TSTORM] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-storm", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_SNOW] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-snow", 16, 0, NULL);
+    weather_pixbufs_mini[PIX_FOG] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-fog", 16, 0, NULL);
 
     weather_pixbufs = g_new(GdkPixbuf *, NUM_PIX);
-    weather_pixbufs[PIX_UNKNOWN] = gdk_pixbuf_new_from_xpm_data ((const char **)unknown_xpm);
-    weather_pixbufs[PIX_SUN] = gdk_pixbuf_new_from_xpm_data ((const char **)sun_xpm);
-    weather_pixbufs[PIX_SUNCLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)suncloud_xpm);
-    weather_pixbufs[PIX_CLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)cloud_xpm);
-    weather_pixbufs[PIX_RAIN] = gdk_pixbuf_new_from_xpm_data ((const char **)rain_xpm);
-    weather_pixbufs[PIX_TSTORM] = gdk_pixbuf_new_from_xpm_data ((const char **)tstorm_xpm);
-    weather_pixbufs[PIX_SNOW] = gdk_pixbuf_new_from_xpm_data ((const char **)snow_xpm);
-    weather_pixbufs[PIX_FOG] = gdk_pixbuf_new_from_xpm_data ((const char **)fog_xpm);
+    weather_pixbufs[PIX_UNKNOWN] = gtk_icon_theme_load_icon (icon_theme, "stock_unknown", 48, 0, NULL);
+    weather_pixbufs[PIX_SUN] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-sunny", 48, 0, NULL);
+    weather_pixbufs[PIX_SUNCLOUD] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-few-clouds", 48, 0, NULL);
+    weather_pixbufs[PIX_CLOUD] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-cloudy", 48, 0, NULL);
+    weather_pixbufs[PIX_RAIN] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-showers", 48, 0, NULL);
+    weather_pixbufs[PIX_TSTORM] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-storm", 48, 0, NULL);
+    weather_pixbufs[PIX_SNOW] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-snow", 48, 0, NULL);
+    weather_pixbufs[PIX_FOG] = gtk_icon_theme_load_icon (icon_theme, "stock_weather-fog", 48, 0, NULL);
 }
 
 void _weather_info_get_pixbuf (WeatherInfo *info, gboolean mini, GdkPixbuf **pixbuf)
@@ -2077,7 +2232,7 @@ void _weather_info_get_pixbuf (WeatherInfo *info, gboolean mini, GdkPixbuf **pix
                     idx = PIX_FOG;
                     break;
                 default:
-                    g_assert_not_reached();
+	            idx = PIX_UNKNOWN;
                 }
         } else {
             switch (sky) {
@@ -2093,7 +2248,7 @@ void _weather_info_get_pixbuf (WeatherInfo *info, gboolean mini, GdkPixbuf **pix
                 idx = PIX_CLOUD;
                 break;
             default:
-                g_assert_not_reached();
+	        idx = PIX_UNKNOWN;
             }
         }
     }
